unit fOpenAIEmbeddings;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  // sgc
  sgcAI_OpenAI, sgcAI_OpenAI_Embeddings, sgcAI, sgcBase_Classes,
  Vcl.ComCtrls, sgcAI_DB_Vector, sgcAI_DB_Vector_File, sgcAI_DB_Vector_Pinecone;

type
  TFRMOpenAIEmbeddings = class(TForm)
    pnlTop: TPanel;
    pnlClient: TPanel;
    memoLog: TMemo;
    btnCreateEmbeddingsFromFile: TButton;
    txtAPIKey: TEdit;
    Label1: TLabel;
    Embeddings: TsgcAIOpenAIEmbeddings;
    ProgressBar1: TProgressBar;
    DBVectorFile: TsgcAIDatabaseVectorFile;
    DBVectorPinecone: TsgcAIDatabaseVectorPinecone;
    PageControlDB: TPageControl;
    tabFile: TTabSheet;
    tabPinecone: TTabSheet;
    txtFileInputFilename: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    txtFileVectorFilename: TEdit;
    lblApiKey: TLabel;
    txtPineconeApiKey: TEdit;
    Label21: TLabel;
    txtPineconeEnvironment: TEdit;
    Label8: TLabel;
    txtPineconeIndexName: TEdit;
    Label9: TLabel;
    txtPineconeProjectId: TEdit;
    cboDatabase: TComboBox;
    Label4: TLabel;
    btnCreateEmbeddingsFromDirectory: TButton;
    procedure btnCreateEmbeddingsFromDirectoryClick(Sender: TObject);
    procedure btnCreateEmbeddingsFromFileClick(Sender: TObject);
    procedure cboDatabaseChange(Sender: TObject);
    procedure txtPineconeApiKeyChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EmbeddingsCreateEmbeddingsStart(Sender: TObject; aSize: Int64);
    procedure EmbeddingsCreateEmbeddingsStop(Sender: TObject);
    procedure sgcAIOpenAIEmbeddings1CreateEmbeddingsProgress(Sender: TObject;
        aPosition: Int64; var aContinue: Boolean);
    procedure txtAPIKeyChange(Sender: TObject);
    procedure txtFileInputFilenameChange(Sender: TObject);
    procedure txtFileVectorFilenameChange(Sender: TObject);
    procedure txtPineconeEnvironmentChange(Sender: TObject);
    procedure txtPineconeIndexNameChange(Sender: TObject);
    procedure txtPineconeProjectIdChange(Sender: TObject);
  private
    procedure DoLoadSettings;
    procedure DoSaveSettings;
  end;

var
  FRMOpenAIEmbeddings: TFRMOpenAIEmbeddings;

implementation

{$R *.dfm}

uses
  INIFiles;

procedure TFRMOpenAIEmbeddings.btnCreateEmbeddingsFromDirectoryClick(Sender:
    TObject);
var
  oDialog: TOpenDialog;
  oSearch: TSearchRec;
  vPath: string;
begin
  oDialog := TOpenDialog.Create(nil);
  Try
    if oDialog.Execute then
    begin
      vPath := IncludeTrailingPathDelimiter(ExtractFileDir(oDialog.FileName));
      if FindFirst(vPath + '*.*', faArchive, oSearch) = 0 then
      begin
        repeat
          Embeddings.CreateEmbeddingsFromFile(vPath + oSearch.Name);
        until FindNext(oSearch) <> 0;

        FindClose(oSearch);
      end;
    end;
  Finally
    FreeAndNil(oDialog);
  End;
end;

procedure TFRMOpenAIEmbeddings.btnCreateEmbeddingsFromFileClick(Sender: TObject);
var
  oDialog: TOpenDialog;
begin
  case cboDatabase.ItemIndex of
    1: Embeddings.Database := DBVectorPinecone;
    else
      Embeddings.Database := DBVectorFile;
  end;

  oDialog := TOpenDialog.Create(nil);
  Try
    oDialog.Filter := 'TXT Files|*.txt';
    if oDialog.Execute then
      Embeddings.CreateEmbeddingsFromFile(oDialog.FileName);
  Finally
    FreeAndNil(oDialog);
  End;
end;

procedure TFRMOpenAIEmbeddings.cboDatabaseChange(Sender: TObject);
begin
  pageControlDB.ActivePageIndex := cboDatabase.ItemIndex;
end;

procedure TFRMOpenAIEmbeddings.FormDestroy(Sender: TObject);
begin
  DoSaveSettings;
end;

procedure TFRMOpenAIEmbeddings.FormCreate(Sender: TObject);
var
  i: integer;
begin
  DoLoadSettings;

  for i := 0 to PageControlDB.PageCount - 1 do
    PageControlDB.Pages[i].TabVisible := false;
  PageControlDB.ActivePageIndex := cboDatabase.ItemIndex;
end;

procedure TFRMOpenAIEmbeddings.DoLoadSettings;
var
  oINI: TINIFile;
begin
  oINI := TINIFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  Try
    txtApiKey.Text := oINI.ReadString('OPENAI', 'API_KEY', '');

    cboDatabase.ItemIndex := oINI.ReadInteger('DB', 'TYPE', 0);

    txtFileInputFilename.Text := oINI.ReadString('DB.FILE', 'INPUT_FILENAME', 'sgcEmbeddings.sgcif');
    txtFileVectorFilename.Text := oINI.ReadString('DB.FILE', 'VECTOR_FILENAME', 'sgcEmbeddings.sgcvf');

    txtPineconeApiKey.Text := oINI.ReadString('DB.PINECONE', 'API_KEY', '');
    txtPineconeEnvironment.Text := oINI.ReadString('DB.PINECONE', 'ENVIRONMENT', 'us-west4-gcp-free');
    txtPineconeIndexName.Text := oINI.ReadString('DB.PINECONE.INDEX', 'INDEX_NAME', '');
    txtPineconeProjectId.Text := oINI.ReadString('DB.PINECONE.INDEX', 'PROJECT_ID', '');
  Finally
    oINI.Free;
  End;
end;

procedure TFRMOpenAIEmbeddings.DoSaveSettings;
var
  oINI: TINIFile;
begin
  oINI := TINIFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  Try
    oINI.Writestring('OPENAI', 'API_KEY', txtApiKey.Text);

    oINI.WriteInteger('DB', 'TYPE', cboDatabase.ItemIndex);

    oINI.Writestring('DB.FILE', 'INPUT_FILENAME', txtFileInputFilename.Text);
    oINI.Writestring('DB.FILE', 'VECTOR_FILENAME', txtFileVectorFilename.Text);

    oINI.Writestring('DB.PINECONE', 'API_KEY', txtPineconeApiKey.Text);
    oINI.Writestring('DB.PINECONE', 'ENVIRONMENT', txtPineconeEnvironment.Text);
    oINI.WriteString('DB.PINECONE.INDEX', 'INDEX_NAME', txtPineconeIndexName.Text);
    oINI.WriteString('DB.PINECONE.INDEX', 'PROJECT_ID', txtPineconeProjectId.Text);
  Finally
    oINI.Free;
  End;
end;

procedure TFRMOpenAIEmbeddings.txtPineconeApiKeyChange(Sender: TObject);
begin
  DBVectorPinecone.PineconeOptions.ApiKey := txtPineconeApiKey.Text;
end;

procedure TFRMOpenAIEmbeddings.EmbeddingsCreateEmbeddingsStart(Sender: TObject;
    aSize: Int64);
begin
  ProgressBar1.Max := aSize;
  ProgressBar1.Position := 0;
end;

procedure TFRMOpenAIEmbeddings.EmbeddingsCreateEmbeddingsStop(Sender: TObject);
begin
  ProgressBar1.Position := 0;
  ProgressBar1.Visible := False;
end;

procedure TFRMOpenAIEmbeddings.sgcAIOpenAIEmbeddings1CreateEmbeddingsProgress(
    Sender: TObject; aPosition: Int64; var aContinue: Boolean);
begin
  ProgressBar1.Position := aPosition;
  Application.ProcessMessages;
end;

procedure TFRMOpenAIEmbeddings.txtAPIKeyChange(Sender: TObject);
begin
  Embeddings.OpenAIOptions.ApiKey := txtAPIKey.Text;
end;

procedure TFRMOpenAIEmbeddings.txtFileInputFilenameChange(Sender: TObject);
begin
  DBVectorFile.VectorFileOptions.InputFilename := txtFileInputFilename.Text;
end;

procedure TFRMOpenAIEmbeddings.txtFileVectorFilenameChange(Sender: TObject);
begin
  DBVectorFile.VectorFileOptions.VectorFilename := txtFileVectorFilename.Text;
end;

procedure TFRMOpenAIEmbeddings.txtPineconeEnvironmentChange(Sender: TObject);
begin
  DBVectorPinecone.PineconeOptions.Environment := txtPineconeEnvironment.Text;
end;

procedure TFRMOpenAIEmbeddings.txtPineconeIndexNameChange(Sender: TObject);
begin
  DBVectorPinecone.PineconeIndexOptions.IndexName := txtPineconeIndexName.Text;
end;

procedure TFRMOpenAIEmbeddings.txtPineconeProjectIdChange(Sender: TObject);
begin
  DBVectorPinecone.PineconeIndexOptions.ProjectId := txtPineconeProjectId.Text;
end;

end.
